﻿--  ===============================
--	Tabard Addict - TabardAddict.lua
--	By: gmz323(Greg)
--  ===============================

-- MOP Changes
-- ==============
-- Changed achievement id sent in GetAchievementInfo to 621 (not working with the others)
-- Removed 2nd lookup to GetAchievementInfoByID - GAI returning true/false correctly

-- WOD Changes
-- ==============
-- configs updated to save true/false - due to api change of SetChecked/GetChecked
-- add warlords of draenai config


-- Constants
local TABARD_ADDICT_VERSION = "2.27"
local TABARD_ADDICT_DATA_ROWS = 115

-- Locals
local TabardAddictServerData = {} -- equip data from server
local TabardAddictServerData_tooltiptable = {} -- table for tooltip display
local TabardAddictDoServerQuery;
local TabardAddictUpdateSuggestTable;
local TabardAddictUpdateDisplayTable;

local CurrentTabardEntries_itemID = {};
local CurrentTabardEntries_itemLink = {};
local CurrentTabardSuggest_itemID = {}; 
local CurrentTabardSuggest_itemLink = {};
local InfoFrameCurrent_itemID = 0;

local tempDisplayData = {} -- temp table built to show subset of data
local tempDisplayData_rowcount;
local tempSuggestData = {} -- temp table built to show subset of data
local tempSuggestData_rowcount;
local tempAchievementTooltipData = {} -- table holding achievement tooltip data
local tabardsEquipped = 0;
local taNameFilter = "";
local taLineAdded = false;
local taPlayerFaction = 0; -- 1=alliance, 2=horde, 3=neutral
local taPlayerLevel = 0;
local taSuggestString = "";

-- Localized Strings
local L = TabardAddictLocalization;

-- config table defaults - saved per character
local TabardAddictConfigDefaults = 
{
	CB_AO = true,
	CB_HO = true,
	CB_EQ = true,
	CB_TCG = true,
	CB_Other = true,
	CB_NA = true,
	CB_PVP = true,
	CB_BCF = true,
	CB_WotLKF = true,
	CB_ATF = true,
	CB_CF = true,
	CB_MOPF = true,
	CB_WOD = true,
	CB_SUGG_ACH = true,
	CB_SUGG_QST = true,
	CB_SUGG_CUR = true,
	CB_INFO_EQUIP = true,
	};

-- Slash commands
SLASH_TABARDADDICT1 = "/ta"
SLASH_TABARDADDICT2 = "/tabardaddict"
BINDING_HEADER_TABARDADDICT = L.ADDON_TITLE;
BINDING_NAME_TABARDADDICT_TOGGLE = L.LAUNCHER_TT;


SlashCmdList["TABARDADDICT"] = function(msg, editbox)
	if frameTabardAddict:IsShown() then
		frameTabardAddict:Hide();
	else
		frameTabardAddict:Show();
	end
end

-- Debugging
local debugMode = false;

-- copied from ModelFrames.lua
local _, playerRaceSex = UnitRace("player");
do
	if ( UnitSex("player") == 2 ) then
		playerRaceSex = playerRaceSex.."Male";
	else
		playerRaceSex = playerRaceSex.."Female";
	end
end
	
--*****************************************************************************
--*****************************************************************************
local function DressUpTexturePath(raceFileName)
    -- Orc texture as default
    if ( not raceFileName ) then
    	raceFileName = "Orc";
    end

    return "Interface\\DressUpFrame\\DressUpBackground-"..raceFileName;
end
 
--*****************************************************************************
--***************************************************************************** 
function SetTADressUpBackground()
	local race, fileName = UnitRace("player");
	local texture = DressUpTexturePath(fileName);
	TADressUpBackgroundTop:SetTexture(texture..1);
end

--*****************************************************************************
--***************************************************************************** 
local function TabardAddict_CheckPlayerFaction()

	-- set alliance or horde background
	local englishFaction = UnitFactionGroup("player");
	if ( englishFaction == "Horde" ) then  -- horde
		TabardAddictBGFaction:Show();
		TabardAddictBGFaction2:Hide();
		TabardAddictBGFactionSugg:Show();
		TabardAddictBGFactionSugg2:Hide();
		taPlayerFaction = 2;
	elseif ( englishFaction == "Alliance" ) then
		TabardAddictBGFaction:Hide();
		TabardAddictBGFaction2:Show();
		TabardAddictBGFactionSugg:Hide();
		TabardAddictBGFactionSugg2:Show();
		taPlayerFaction = 1;
	else  -- Neutral faction - Pandaren 1-10
		TabardAddictBGFaction:Hide();
		TabardAddictBGFaction2:Hide();
		TabardAddictBGFactionSugg:Hide();
		TabardAddictBGFactionSugg2:Hide();
		taPlayerFaction = 3;
	end;

end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_GetStandingTxt(standingID)
	local standingTxt;
	
	local gender = UnitSex("player");

	if (standingID >= 1 and standingID <= 8) then
		if (gender == 3 ) then --female - important for non-us locale
			standingTxt = _G['FACTION_STANDING_LABEL'..standingID..'_FEMALE'];
		else
			standingTxt = _G['FACTION_STANDING_LABEL'..standingID];
		end;
	else
		standingTxt = "";
	end;
	
	return standingTxt;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_BuildTooltipData()

	-- 621 - first tabard achievement
	local achievementID = 621;
	local i = 1;

	if (debugMode) then
		print("BuildTooltipData");
	end;
	
	while (achievementID ~= nil)
	do
		if (debugMode) then
			print("achievementID: " .. tostring(achievementID));
		end;
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch = GetAchievementInfo(achievementID);
		if (id ~= nil) then
			tempAchievementTooltipData[i] = {};
			tempAchievementTooltipData[i][1] = completed;
			tempAchievementTooltipData[i][2] = name;
			i = i + 1;
			local nextID, completed = GetNextAchievement(id);
			achievementID = nextID;
		else
			achievementID = nil;
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckReputationRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- get reputation for this tabards faction -faction id in taTabardReqmtData[itemID][3]
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfoByID(taTabardReqmtData[itemID][3]);
	
	-- handle bad faction lookup
	if (name == nil) then
	  return 0;
	end;
	
	-- compare against required level - level in [4]
	if (standingID >= taTabardReqmtData[itemID][4]) then
		taSuggestString = string.format(FACTION_STANDING_CHANGED, TabardAddict_GetStandingTxt(taTabardReqmtData[itemID][4]), name);
		return 1;
	else
		return 0;
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckQuestRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- get quest name from questid - taTabardReqmtData[itemID][3]
	-- get quest level from - taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player");
	
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		if (1) then --quest config
			taSuggestString = L.SUGG_QUEST_TXT .. " " .. taTabardReqmtData[itemID][3];
			return 1;
		else
			return 0;
		end;
	end;
	return 0;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckAchievementRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- get achievement name from achievementid - taTabardReqmtData[itemID][3]
	-- get achievement level from taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player");

	-- achievement levels are currently 0 - update!!!
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		-- get achievement name
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch = GetAchievementInfo(taTabardReqmtData[itemID][3]);
		if (name == nil) then
			return 0;
		end;
		taSuggestString = L.SUGG_ACHV_TXT .. name;
		return 1;
	else
		return 0;
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckCurrencyRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- currency amount and name string - taTabardReqmtData[itemID][3]
	-- required level - taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player");
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		taSuggestString = L.SUGG_CURRENCY_TXT .. taTabardReqmtData[itemID][3];
		return 1;
	else
		return 0;
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckDropRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- mob id taTabardReqmtData[itemID][3]
	-- mob level taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player");
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		-- set taSuggestString
		taSuggestString = L.SUGG_DROP_TXT .. taTabardReqmtData[itemID][3] .. " (" .. LEVEL_ABBR .. " " .. taTabardReqmtData[itemID][4] .. ")";
		return 1;
	else
		return 0;
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CheckSpecialRequirement(itemID)

	-- handle the midsummer event tabards - 35279, 35280
	-- if one has been equipped dont suggest the other
	if (itemID == 35279) then
		if (TabardAddictServerData[35280] == true) then
			return 0; -- can't equip both tabards
		else
			return 1;
		end;
	end;
	if (itemID == 35280) then
		if (TabardAddictServerData[35279] == true) then
			return 0; -- can't equip both tabards
		else
			return 1;
		end;
	end;
	
	-- handle paladin blood elf
	if (itemID == 25549) then
		local class, classFileName = UnitClass("player");
		local race, raceFileName = UnitRace("player");
		
		if ( (strupper(classFileName) == "PALADIN") and (strupper(raceFileName) == "BLOODELF") ) then
			return 1;
		else
			return 0;
		end;
	end;
	
	-- handle draenei only
	if (itemID == 24344) then
		local race, raceFileName = UnitRace("player");
		
		if (strupper(raceFileName) == "DRAENEI") then
			return 1;
		else
			return 0;
		end;
	end;
	
	-- handle Battle of the Crimson Watch rewards
	-- if one has been equipped dont suggest the other
	if (itemID == 31404) then
		if (TabardAddictServerData[31405] == true) then
			return 0; -- can't equip both tabards
		else
			return 1;
		end;
	end;
	if (itemID == 31405) then
		if (TabardAddictServerData[31404] == true) then
			return 0; -- can't equip both tabards
		else
			return 1;
		end;
	end;
	
	return 1;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_PopulateDisplayTable()
	
	-- if the suggest table is nil force an update
	if (tempDisplayData == nil) then
		TabardAddictUpdateDisplayTable = true;
	end;

	-- Only need to populate this table once - and when server data is rechecked or config change
	if (TabardAddictUpdateDisplayTable == true) then

		local maintableRow, j, displayThisRow;
		wipe(tempDisplayData);
		tempDisplayData_rowcount = 0;
		TabardAddictUpdateDisplayTable = false;

		if (debugMode) then
			print ("Tabard Addict: PopulateDisplayTable() called");
			for index,value in pairs(TabardAddictConfig_v2) do
				print(tostring(index) .. " : " .. tostring(value));
			end;	
		end;
		
		-- This creates a temp table by joining local tabard data and server results
		for maintableRow in ipairs(taTabardData)
		do
			-- determine if this row should be displayed -- by Tabard Type
			if ((taTabardData[maintableRow][5] == 0) and (TabardAddictConfig_v2.CB_Other == true)) then
				displayThisRow = 1;
			elseif ((taTabardData[maintableRow][5] == 1) and (TabardAddictConfig_v2.CB_PVP == true)) then
				displayThisRow = 1; 	
			elseif ((taTabardData[maintableRow][5] == 2) and (TabardAddictConfig_v2.CB_Other == true)) then
				displayThisRow = 1; 	
			elseif ((taTabardData[maintableRow][5] == 3) and (TabardAddictConfig_v2.CB_TCG == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 4) and (TabardAddictConfig_v2.CB_NA == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 6) and (TabardAddictConfig_v2.CB_BCF == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 7) and (TabardAddictConfig_v2.CB_WotLKF == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 8) and (TabardAddictConfig_v2.CB_ATF == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 9) and (TabardAddictConfig_v2.CB_CF == true)) then
				displayThisRow = 1; 
			elseif ((taTabardData[maintableRow][5] == 10) and (TabardAddictConfig_v2.CB_MOPF == true)) then
				displayThisRow = 1; 	
			elseif ((taTabardData[maintableRow][5] == 11) and (TabardAddictConfig_v2.CB_WOD == true)) then
				displayThisRow = 1; 	
			else
				displayThisRow = 0;
			end;
			
			-- determine if this row should be filtered -- Filter out by Alliance/Horde
			if ((taTabardData[maintableRow][6] == 1) and (TabardAddictConfig_v2.CB_AO == false)) then
				displayThisRow = 0;
			elseif ((taTabardData[maintableRow][6] == 2) and (TabardAddictConfig_v2.CB_HO == false)) then
				displayThisRow = 0; 
			end;
			-- hide equipped tabards
			local iID = taTabardData[maintableRow][1];
			if (iID ~= nil) then
				if ( (TabardAddictServerData[iID] == true ) and (TabardAddictConfig_v2.CB_EQ == false)) then
					displayThisRow = 0; 
				end;
			end;
			
			-- handle search box
			if ((displayThisRow == 1) and (taNameFilter ~= "")) then
				if( string.find(string.lower(taTabardData[maintableRow][2]),string.lower(taNameFilter)) == nil ) then
					displayThisRow = 0;
				end;
			end;
			
			if (displayThisRow == 1) then
				tempDisplayData_rowcount = tempDisplayData_rowcount + 1;
				tempDisplayData[tempDisplayData_rowcount] = {};     -- create a new row
				for j=1,4 do -- copy first 4 column
					tempDisplayData[tempDisplayData_rowcount][j] = taTabardData[maintableRow][j];
				end
				-- column 5 - init equipped bool
				tempDisplayData[tempDisplayData_rowcount][5] = false;
				-- column 6 - tabard type
				tempDisplayData[tempDisplayData_rowcount][6] = taTabardData[maintableRow][5];
				-- column 7 - alliance or horde indicator
				tempDisplayData[tempDisplayData_rowcount][7] = taTabardData[maintableRow][6];
			    -- column 8 - criteria ID
				tempDisplayData[tempDisplayData_rowcount][8] = taTabardData[maintableRow][7];
				
				
				-- set equipped bool from server data
				local iID = taTabardData[maintableRow][1];
				if (iID ~= nil) then
					tempDisplayData[tempDisplayData_rowcount][5] = TabardAddictServerData[iID];
				end;
				
			end;
	    end;
		TabardAddictTab1.taTabardTotalShowing:SetText(string.format(L.TABARDS_SHOWN, tempDisplayData_rowcount));
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_PopulateSuggestionsTable()

	-- if the suggest table is nil force an update
	if (tempSuggestData == nil) then
		TabardAddictUpdateSuggestTable = true;
	end;
	
	-- Only need to populate this table once - and when server date is rechecked or config change
	if (TabardAddictUpdateSuggestTable == true) then
	
		if (debugMode) then
			print("Tabard Addict: PopulateSuggestionsTable() called");
		end;
	
		local maintableRow, j, suggestThisRow;
		wipe(tempSuggestData);
		tempSuggestData_rowcount = 0;
		TabardAddictUpdateSuggestTable = false;
		
		-- Check Alliance/Horde/Neutral
		TabardAddict_CheckPlayerFaction();
		
		-- This creates a temp table by joining local tabard data and server results
		for maintableRow in ipairs(taTabardData)
		do
			-- init var
			suggestThisRow = 0;
			--get item id
			local iID = taTabardData[maintableRow][1];
			
			-- check if been equipped
			if (TabardAddictServerData[iID] == false) then
				suggestThisRow = 1;
			end;
			-- check if currently available 
			if (suggestThisRow == 1) then
				if (taTabardReqmtData[iID][0] ~= 1) then
					suggestThisRow = 0;
				end;
			end;
			-- check alliance(1) and horde(2) factions
			if (suggestThisRow == 1) then
				if (taTabardReqmtData[iID][1] == 1 and taPlayerFaction ~= 1 ) then
					suggestThisRow = 0;
				elseif (taTabardReqmtData[iID][1] == 2 and taPlayerFaction ~= 2 ) then
					suggestThisRow = 0;
				else
					suggestThisRow = 1;
				end;
			end;
			
			-- init Suggestion String
			taSuggestString = "";
			
			-- check requirements
			if (suggestThisRow == 1) then
				-- check the rep/quest/etc requirements
				if (taTabardReqmtData[iID][2] == 0) then
					suggestThisRow = 1;
					taSuggestString = L.SUGG_NOREQ_TXT;
				elseif (taTabardReqmtData[iID][2] == 1) then
					-- requires reputation 
					suggestThisRow = TabardAddict_CheckReputationRequirement(iID);
				elseif (taTabardReqmtData[iID][2] == 2 and TabardAddictConfig_v2.CB_SUGG_QST == true) then
					-- requires quest completion
					suggestThisRow = TabardAddict_CheckQuestRequirement(iID);
				elseif (taTabardReqmtData[iID][2] == 3 and TabardAddictConfig_v2.CB_SUGG_ACH == true) then
					-- requires achievement completion
					suggestThisRow = TabardAddict_CheckAchievementRequirement(iID);
				elseif (taTabardReqmtData[iID][2] == 4 and TabardAddictConfig_v2.CB_SUGG_CUR == true) then
					-- requires currency to purchase
					suggestThisRow = TabardAddict_CheckCurrencyRequirement(iID);
				elseif (taTabardReqmtData[iID][2] == 5) then
					-- requires drop from mob
					suggestThisRow = TabardAddict_CheckDropRequirement(iID);
				else
					suggestThisRow = 0;
				end;
			end;
			
			-- handle special cases - class/race tabards, event tabards, etc...
			if (suggestThisRow == 1) then
				if (taTabardReqmtSpecial[iID]) then
					suggestThisRow = TabardAddict_CheckSpecialRequirement(iID);
				end;
			end;
			
			-- add to table if passed all the checks
			if (suggestThisRow == 1) then
				tempSuggestData_rowcount = tempSuggestData_rowcount + 1;
				tempSuggestData[tempSuggestData_rowcount] = {};     -- create a new row
				for j=1,4 do -- copy first 4 column
					tempSuggestData[tempSuggestData_rowcount][j] = taTabardData[maintableRow][j];
				end;
				-- column 5 - been equipped - must be false
				tempSuggestData[tempSuggestData_rowcount][5] = false;
				-- column 6 - tabard type
				tempSuggestData[tempSuggestData_rowcount][6] = taTabardData[maintableRow][5];
				-- column 7 - alliance or horde indicator
				tempSuggestData[tempSuggestData_rowcount][7] = taTabardData[maintableRow][6];
				-- column 8 - suggestion test string to be displayed
				tempSuggestData[tempSuggestData_rowcount][8] = taSuggestString;
			end;
		end;
	end;
	TabardAddictTab4.taTabardTotalShowing:SetText(string.format(L.TABARDS_SUGGESTED, tempSuggestData_rowcount));
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_SortDisplayTable()

	-- currently sorting only on tabard name
	table.sort(tempDisplayData, function(a,b) return a[2]<b[2] end);
	
end;
--
local function TabardAddict_SortSuggestionsTable()

	-- currently sorting only on tabard name
	table.sort(tempSuggestData, function(a,b) return a[2]<b[2] end);
	
end;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_CleanConfigVars()
	
	local key, value;

	-- loop through the client stored variables - remove those no longer used
	for key,value in pairs(TabardAddictConfig_v2) do 
		if (TabardAddictConfigDefaults[key] == nil) then
			TabardAddictConfig_v2[key] = nil;
			if (debugMode) then
				print("Tabard Addict - Removing old config value: " .. key);
			end;
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnEvent(self, event, ...)
	
	
	-- Check if player changed tabard
	if (event == "PLAYER_EQUIPMENT_CHANGED") then
		
		local slot, hasItem = ...;
		if hasItem == nil then 
			hasItem = 0;
		end;
		
		-- addon is shown - do an update and return to tab1
		if frameTabardAddict:IsShown() then	
			-- if tabard slot and item put into slot - refresh listing and count
			if ( (slot == INVSLOT_TABARD) and (hasItem == 1) ) then
				TabardAddictInfoFrame:Hide();
				TabardAddictDoServerQuery = true;
				TabardAddict_OnShow(self);
			end;
			if (debugMode) then
				print("Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. hasItem);
			end;
		else
			-- addon not shown - updated on next show event
			if ( (slot == INVSLOT_TABARD) and (hasItem == 1) ) then
				TabardAddictInfoFrame:Hide();
				TabardAddictDoServerQuery = true;
			end;
			if (debugMode) then
				print("Not Shown - Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. hasItem);
			end;
		end;
	end;
	if (event == "ADDON_LOADED") then
		local arg1 = ...;
		
		if (arg1 == "TabardAddict") then
			--Show/Hide character frame launcher
			frameTabardAddictCharFrameLauncher:Show();
			
			-- check for nil global config values - if not exist then default to on (true)
			if (TabardAddictConfig_v2 == nil) then
				TabardAddictConfig_v2 = TabardAddictConfigDefaults;
			end;
			-- check for nil on each config item - in case a new one is added
			if (TabardAddictConfig_v2.CB_AO == nil) then
				TabardAddictConfig_v2.CB_AO = TabardAddictConfigDefaults.CB_AO;
			end;
			if (TabardAddictConfig_v2.CB_HO == nil) then
				TabardAddictConfig_v2.CB_HO  = TabardAddictConfigDefaults.CB_HO;
			end;
			if (TabardAddictConfig_v2.CB_EQ == nil) then
				TabardAddictConfig_v2.CB_EQ  = TabardAddictConfigDefaults.CB_EQ;
			end;
			if (TabardAddictConfig_v2.CB_TCG == nil) then
				TabardAddictConfig_v2.CB_TCG = TabardAddictConfigDefaults.CB_TCG;
			end;
			if (TabardAddictConfig_v2.CB_Other == nil) then
				TabardAddictConfig_v2.CB_Other = TabardAddictConfigDefaults.CB_Other;
			end;
			if (TabardAddictConfig_v2.CB_NA == nil) then
				TabardAddictConfig_v2.CB_NA = TabardAddictConfigDefaults.CB_NA;
			end;
			if (TabardAddictConfig_v2.CB_PVP == nil) then
				TabardAddictConfig_v2.CB_PVP = TabardAddictConfigDefaults.CB_PVP;
			end;
			if (TabardAddictConfig_v2.CB_BCF == nil) then
				TabardAddictConfig_v2.CB_BCF = TabardAddictConfigDefaults.CB_BCF;
			end;
			if (TabardAddictConfig_v2.CB_WotLKF == nil) then
				TabardAddictConfig_v2.CB_WotLKF = TabardAddictConfigDefaults.CB_WotLKF;
			end;
			if (TabardAddictConfig_v2.CB_ATF == nil) then
				TabardAddictConfig_v2.CB_ATF = TabardAddictConfigDefaults.CB_ATF;
			end;
			if (TabardAddictConfig_v2.CB_CF == nil) then
				TabardAddictConfig_v2.CB_CF = TabardAddictConfigDefaults.CB_CF;
			end;
			if (TabardAddictConfig_v2.CB_MOPF == nil) then
				TabardAddictConfig_v2.CB_MOPF = TabardAddictConfigDefaults.CB_MOPF;
			end;
			if (TabardAddictConfig_v2.CB_WOD == nil) then
				TabardAddictConfig_v2.CB_WOD = TabardAddictConfigDefaults.CB_WOD;
			end;
			if (TabardAddictConfig_v2.CB_SUGG_ACH == nil) then
				TabardAddictConfig_v2.CB_SUGG_ACH = TabardAddictConfigDefaults.CB_SUGG_ACH;
			end;
			if (TabardAddictConfig_v2.CB_SUGG_QST == nil) then
				TabardAddictConfig_v2.CB_SUGG_QST = TabardAddictConfigDefaults.CB_SUGG_QST;
			end;
			if (TabardAddictConfig_v2.CB_SUGG_CUR == nil) then
				TabardAddictConfig_v2.CB_SUGG_CUR = TabardAddictConfigDefaults.CB_SUGG_CUR;
			end;
			if (TabardAddictConfig_v2.CB_INFO_EQUIP == nil) then
				TabardAddictConfig_v2.CB_INFO_EQUIP = TabardAddictConfigDefaults.CB_INFO_EQUIP;
			end;
			
			-- cleanup old config vars
			TabardAddict_CleanConfigVars();
			
		end;
	end;
	if (event == "UPDATE_FACTION") then
		-- do server query in case player has gained enough rep for a new tabard
		TabardAddictDoServerQuery = true;
	end;
end
 
--*****************************************************************************
--*****************************************************************************
function TabardAddict_GetServerData() 

	local j,m,n,r,a,b;
	
	wipe(TabardAddictServerData);
	
	-- force update of Display table
	TabardAddictUpdateDisplayTable = true;
	-- force update of Suggest table
	TabardAddictUpdateSuggestTable = true;
	
	-- id 1020 = ten tabards achievement
	-- id 621 = represent - MOP UPDATE - query not working with other tabard achievements 
	for b=1,TABARD_ADDICT_DATA_ROWS
	do 
		-- failing as of 11/28 5.1 client
		-- TODO - recheck in future
		--_,_,_,_,_,_,_,_,_,j = GetAchievementCriteriaInfo(621,b);
		-- workaround to lookup first 98 tabards
		-- added new criteria ids 12/08/2012
		j = taTabardData[b][7];
		
		-- Only lookup when creteriaID is not zero - else save as false
		if (j > 0) then
			n,_,m,_,_,_,_,r,_,_,_,_,_ = GetAchievementCriteriaInfoByID(621,j);
		else
			r = taTabardData[b][1];
			m = 'na'; -- this will cause tooltip to show 'not tracked'
		end;
		TabardAddictServerData[r] = m; -- save "criteria met" true/false
		-- update tooltip table
		TabardAddictServerData_tooltiptable[r] = m;  -- save "criteria met" true/false
		if (m == true) then
			-- increment total tabards equipped
			tabardsEquipped = tabardsEquipped + 1;
		end;
	end;
	-- Build tooltip data
	TabardAddict_BuildTooltipData();
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictScrollBar_Update()
	local line; -- 1 through 8 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(TabardAddictScrollBar,tempDisplayData_rowcount,8,40);
	for line=1,8 
	do
		lineplusoffset = line + FauxScrollFrame_GetOffset(TabardAddictScrollBar);
		
		-- set alpha values for each row
		if (tempDisplayData_rowcount == 0) then
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:Hide();
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:Hide();
		else
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:Show();
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:Show();
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:SetAlpha(0.35);
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:SetAlpha(0.70);
		end;
		if lineplusoffset <= tempDisplayData_rowcount then
			getglobal("TabardAddictEntry"..line):Enable();
			getglobal("TabardAddictEntry"..line).TabardName:SetText(tempDisplayData[lineplusoffset][2]);
			getglobal("TabardAddictEntry"..line).TabardName:Show();
			
			-- check if equipped
			if (tempDisplayData[lineplusoffset][5] == true) then
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cff00FF00"..L.YES.."|r");
				-- show overlay on equipped tabards
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Show(); 
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:SetDrawLayer("OVERLAY",5);
			else
				-- check if criteria id is zero - not counted toward achievement
				if (tempDisplayData[lineplusoffset][8] == 0) then
					getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cff808080"..L.NOT_TRACKED.."|r");
				else
					getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cffFF0000"..L.NO.."|r");
				end;
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide();
			end
			getglobal("TabardAddictEntry"..line).TabardEquipped:Show();
			
			-- set tabard icon texture
			getglobal("TabardAddictEntry"..line).TabardIcon:SetTexture(tempDisplayData[lineplusoffset][4]);
			getglobal("TabardAddictEntry"..line).TabardIcon:Show();
			
			-- set local vars
			CurrentTabardEntries_itemID[line] = tempDisplayData[lineplusoffset][1];
			CurrentTabardEntries_itemLink[line] = tempDisplayData[lineplusoffset][3];
			
			-- set faction icon
			if (tempDisplayData[lineplusoffset][7] == 2) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png");
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			elseif (tempDisplayData[lineplusoffset][7] == 1) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png");
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			else
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide();
			end;
			
		else
			getglobal("TabardAddictEntry"..line):Disable();
			getglobal("TabardAddictEntry"..line).TabardName:Hide();
			getglobal("TabardAddictEntry"..line).TabardEquipped:Hide();
			getglobal("TabardAddictEntry"..line).TabardIcon:Hide();
			getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide();
			getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide();
			-- set local vars
			CurrentTabardEntries_itemID[line] = 0;
			CurrentTabardEntries_itemLink[line] = 0;
			
			-- warn user
			if ((tempDisplayData_rowcount == 0) and (line == 2)) then
				getglobal("TabardAddictEntry"..line).TabardName:SetText(L.ERROR1);
				getglobal("TabardAddictEntry"..line).TabardName:Show();
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.ERROR1A);
				getglobal("TabardAddictEntry"..line).TabardEquipped:Show();
			end;

		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictScrollBar2_Update()
	
	local line; -- 1 through 8 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(TabardAddictScrollBar2,tempSuggestData_rowcount,8,40);

	for line=1,8 
	do
		lineplusoffset = line + FauxScrollFrame_GetOffset(TabardAddictScrollBar2);
		
		-- set alpha values for each row
		getglobal("TabardAddictSugg"..line).TabardAddictButtonBack:SetAlpha(0.35);
		getglobal("TabardAddictSugg"..line).TabardAddictButtonHighlight:SetAlpha(0.70);
		
		if (lineplusoffset <= tempSuggestData_rowcount) then
			getglobal("TabardAddictSugg"..line):Enable();
			getglobal("TabardAddictSugg"..line).TabardName:SetText(tempSuggestData[lineplusoffset][2]);
			getglobal("TabardAddictSugg"..line).TabardName:Show();

			-- set suggestion text
			local displayStr = tempSuggestData[lineplusoffset][8];
			getglobal("TabardAddictSugg"..line).TabardEquipped:SetText(displayStr);
			getglobal("TabardAddictSugg"..line).TabardEquipped:Show();
			
			-- set tabard icon texture
			getglobal("TabardAddictSugg"..line).TabardIcon:SetTexture(tempSuggestData[lineplusoffset][4]);
			getglobal("TabardAddictSugg"..line).TabardIcon:Show();
			
			-- set overlay texture??
			--getglobal("TabardAddictSugg"..line).TabardIconSuggOverlay:Show();
			
			-- set local vars
			CurrentTabardSuggest_itemID[line] = tempSuggestData[lineplusoffset][1];
			CurrentTabardSuggest_itemLink[line] = tempSuggestData[lineplusoffset][3];
			
			-- set faction icon
			if (tempSuggestData[lineplusoffset][7] == 2) then
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png");
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			elseif (tempSuggestData[lineplusoffset][7] == 1) then
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png");
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			else
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Hide();
			end;
			
		else
			getglobal("TabardAddictSugg"..line):Disable();
			getglobal("TabardAddictSugg"..line).TabardName:Hide();
			getglobal("TabardAddictSugg"..line).TabardEquipped:Hide();
			getglobal("TabardAddictSugg"..line).TabardIcon:Hide();
			getglobal("TabardAddictSugg"..line).TabardIconOverlay:Hide();
			getglobal("TabardAddictSugg"..line).TabardFactionIcon:Hide();
			-- set local vars
			CurrentTabardSuggest_itemID[line] = 0;
			CurrentTabardSuggest_itemLink[line] = 0;
			
			-- if no suggestions to show - display message to user
			if ((tempSuggestData_rowcount == 0) and (line == 2)) then
				getglobal("TabardAddictSugg"..line).TabardName:SetText(L.ERROR2);
				getglobal("TabardAddictSugg"..line).TabardName:Show();
			end;
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnEnter(line)
	
	if (debugMode) then
		print(tostring(GetTime()) .. " EntryButton - OnEnter handled - line:" .. tostring(line));
	end;

end;
--
function TabardAddictSuggButton_OnEnter(line)

	if (debugMode) then
		print(tostring(GetTime()) .. " SuggButton - OnEnter handled - line:" .. tostring(line));
	end;

end;
--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnLeave()
	ResetCursor();
	GameTooltip:Hide();
	
	if (debugMode) then
		print(tostring(GetTime()) .. " EntryButton - OnLeave handled");
	end;

end;
--
function TabardAddictSuggButton_OnLeave()
	ResetCursor();
	GameTooltip:Hide();
	
	if (debugMode) then
		print(tostring(GetTime()) .. " SuggButton - OnLeave handled");
	end;
	
end;
--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnUpdate(line)
	
	local itemID = CurrentTabardEntries_itemID[line];
	
	-- Show tooltip if over tabard icon
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		-- Hide info frame
		TabardAddictInfoFrame:Hide();
		
		GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120);
		GameTooltip:SetHyperlink("item:"..itemID);
		GameTooltip:Show();
		-- if ctrl is down show inspect cursor else reset it
		if ( IsModifiedClick("DRESSUP") ) then
			ShowInspectCursor();
		else
			ResetCursor();
		end;
		-- reset main display text
		TabardAddictTab1.taDescText:SetText(string.format(L.TABARD_TITLE, UnitName("player")));
	else
		-- not over icon - hide tooltip and reset cursor
		GameTooltip:Hide();
		ResetCursor();

		-- if info window is not shown - show help text
		if (TabardAddictInfoFrame:IsShown() ~= true) then
			GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120);
			GameTooltip:SetText(L.DETAILWINDOW_TT, 1.0, 1.0, 1.0, 0.85, 1);
		end;
	end;
	
end;
--
function TabardAddictSuggButton_OnUpdate(line)

	local itemID = CurrentTabardSuggest_itemID[line];
	
	-- Show tooltip if over tabard icon
	if ( (MouseIsOver(getglobal("TabardAddictSugg"..line).TabardIcon)) and (itemID > 0) ) then
		-- Hide info frame
		TabardAddictInfoFrame:Hide();
		
		GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120);
		GameTooltip:SetHyperlink("item:"..itemID);
		GameTooltip:Show();
		-- if ctrl is down show inspect cursor else reset it
		if ( IsModifiedClick("DRESSUP") ) then
			ShowInspectCursor();
		else
			ResetCursor();
		end;
		-- reset main display text
		TabardAddictTab4.taSuggestDescText:SetText(L.SUGGESTIONS_TITLE);
	else
		-- not over icon - hide tooltip and reset cursor
		GameTooltip:Hide();
		ResetCursor();

		-- if info window is not shown - show help text
		if (TabardAddictInfoFrame:IsShown() ~= true) then
			GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120);
			GameTooltip:SetText(L.DETAILWINDOW_TT, 1.0, 1.0, 1.0, 0.85, 1);
		end;

	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnModifiedClick(line)
	local itemID = CurrentTabardEntries_itemID[line];
	local itemLink = CurrentTabardEntries_itemLink[line];
	
	-- If there was a DRESSUP or CHATLINK click over a tabard icon handle it
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		if ( IsModifiedClick("DRESSUP") ) then
			DressUpItemLink(itemID);
		elseif ( IsModifiedClick("CHATLINK") ) then
			if (ChatEdit_GetActiveWindow()) then
				ChatEdit_InsertLink(itemLink);
			end;
		end;
	end;
end;
--
function TabardAddictSuggButton_OnModifiedClick(line)
	local itemID = CurrentTabardSuggest_itemID[line];
	local itemLink = CurrentTabardSuggest_itemLink[line];
	
	-- If there was a DRESSUP or CHATLINK click over a tabard icon handle it
	if ( (MouseIsOver(getglobal("TabardAddictSugg"..line).TabardIcon)) and (itemID > 0) ) then
		if ( IsModifiedClick("DRESSUP") ) then
			DressUpItemLink(itemID);
		elseif ( IsModifiedClick("CHATLINK") ) then
			if (ChatEdit_GetActiveWindow()) then
				ChatEdit_InsertLink(itemLink);
			end;
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictTab_OnClick(self, tabID)
	if ( not tabID ) then
		tabID = self:GetID();
	end;
	PanelTemplates_SetTab(frameTabardAddict, tabID);

	-- clear search
	if (taNameFilter ~= "") then
		taNameFilter = "";
		TabardAddictTab1.SearchBox:SetText("");
		TabardAddictUpdateDisplayTable = true;
	end
	-- set search widget back to default display
	TabardAddictTab1.SearchBox:Hide();
	TabardAddictTab1.SearchBox.clearButton:Hide();
	TabardAddictTab1.taDescText:Show();

	-- Hide info frame
	if (TabardAddictInfoFrame:IsShown()) then
		TabardAddictInfoFrame:Hide();
	end;	
	
	if ( tabID == 1 ) then
		-- Tabards tab clicked
		TabardAddictScrollBar:Show();
		TabardAddictScrollBar2:Hide();
		TabardAddictTab1:Show();
		TabardAddictTab2:Hide();
		TabardAddictTab3:Hide();
		TabardAddictTab4:Hide();
		-- populate the display table
		TabardAddict_PopulateDisplayTable();
		-- sort the display table
		TabardAddict_SortDisplayTable();
		-- initial display
		TabardAddictScrollBar_Update();
		TabardAddictShowTabards();
		TabardAddictHideSuggestions();
	elseif ( tabID == 2 ) then
		-- Config tab clicked
		TabardAddictScrollBar:Hide();
		TabardAddictScrollBar2:Hide();
		TabardAddictTab1:Hide();
		TabardAddictTab2:Show();
		TabardAddictTab3:Hide();
		TabardAddictTab4:Hide();
		-- set checkbox values
		CheckButtonAO:SetChecked(TabardAddictConfig_v2.CB_AO);
		CheckButtonHO:SetChecked(TabardAddictConfig_v2.CB_HO);
		CheckButtonEQ:SetChecked(TabardAddictConfig_v2.CB_EQ);
		CheckButtonTCG:SetChecked(TabardAddictConfig_v2.CB_TCG);
		CheckButtonOther:SetChecked(TabardAddictConfig_v2.CB_Other);
		CheckButtonNA:SetChecked(TabardAddictConfig_v2.CB_NA);
		CheckButtonPVP:SetChecked(TabardAddictConfig_v2.CB_PVP);
		CheckButtonBCF:SetChecked(TabardAddictConfig_v2.CB_BCF);
		CheckButtonWotLKF:SetChecked(TabardAddictConfig_v2.CB_WotLKF);
		CheckButtonATF:SetChecked(TabardAddictConfig_v2.CB_ATF);
		CheckButtonCF:SetChecked(TabardAddictConfig_v2.CB_CF);
		CheckButtonMOPF:SetChecked(TabardAddictConfig_v2.CB_MOPF);
		CheckButtonWOD:SetChecked(TabardAddictConfig_v2.CB_WOD);
		CheckButtonSuggAch:SetChecked(TabardAddictConfig_v2.CB_SUGG_ACH);
		CheckButtonSuggQst:SetChecked(TabardAddictConfig_v2.CB_SUGG_QST);
		CheckButtonSuggCur:SetChecked(TabardAddictConfig_v2.CB_SUGG_CUR);
		TabardAddictHideTabards();
		TabardAddictHideSuggestions();
	elseif ( tabID == 3 ) then
		-- About tab clicked
		TabardAddictScrollBar:Hide();
		TabardAddictScrollBar2:Hide();
		TabardAddictTab1:Hide();
		TabardAddictTab2:Hide();
		TabardAddictTab3:Show();
		TabardAddictTab4:Hide();
		TabardAddictHideTabards();
		TabardAddictHideSuggestions();
	elseif ( tabID == 4 ) then	
		-- Suggestions tab clicked
		TabardAddictScrollBar:Hide();
		TabardAddictScrollBar2:Show();
		TabardAddictTab1:Hide();
		TabardAddictTab2:Hide();
		TabardAddictTab3:Hide();
		TabardAddictTab4:Show();
		-- populate the suggestions table
		TabardAddict_PopulateSuggestionsTable();
		-- sort the suggestions table
		TabardAddict_SortSuggestionsTable();
		-- initial display
		TabardAddictHideTabards();
		TabardAddictShowSuggestions();
		TabardAddictScrollBar2_Update();
	end;
	PlaySound("igCharacterInfoTab");
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnClick(self, line, section)

	-- Only show info window if click was not over tabard icon
	if ( MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon) ~= true ) then

		PlaySound("igInventoryRotateCharacter");

		if (TabardAddictInfoFrame:IsShown() ~= true) then
			TabardAddictInfoFrame:Show();
			CheckButtonInfoDressUp.fs:SetText(L.CONFIG_INFO_EQUIP);
			CheckButtonInfoDressUp:SetChecked(TabardAddictConfig_v2.CB_INFO_EQUIP);
		end;

		-- Get Item ID
		local itemID = 0, itemName;
		InfoFrameCurrent_itemID = 0;

		if (section == 0) then
			if (line > 0) then
				itemID = CurrentTabardEntries_itemID[line];
				itemName = getglobal("TabardAddictEntry"..line).TabardName:GetText();
			else 
				return;
			end;
		elseif (section == 1) then
			if (line > 0) then
				itemID = CurrentTabardSuggest_itemID[line];
				itemName = getglobal("TabardAddictSugg"..line).TabardName:GetText();
			else 
				return;
			end;
		else -- only section 0 and 1 currently
			return;
		end;

		TabardAddictInfoFrame.TabardName:SetText(itemName);
		TabardAddictInfoFrame.EditBox:SetText(L.ITEM_INFO_URL..tostring(itemID));
		TabardAddictInfoFrame.EditBox:SetFont("Fonts\\ARIALN.TTF", 13);

		-- save Item ID
		InfoFrameCurrent_itemID = itemID;
		
		-- TryOn item
		if (CheckButtonInfoDressUp:GetChecked() == true) then
			TabardAddictDressUpModel:Dress();
		else
			TabardAddictDressUpModel:Undress();
		end;
		TabardAddictDressUpModel:TryOn(itemID);
		
		-- debug
		if (debugMode) then
			local x,y,z = TabardAddictDressUpModel:GetPosition();
			print("InfoFrame:IsShown(): " .. tostring(TabardAddictInfoFrame:IsShown()));
			print("Model x,y,z position = " .. tostring(x) .. ", " .. tostring(y) .. ", " .. tostring(z));
		end;

	end;

end;


--*****************************************************************************
--*****************************************************************************
function TabardAddict_InfoFrameShowEquipment_OnClick(self)
	
	if self.checked then
        PlaySound("igMainMenuOptionCheckBoxOn");
    else 
        PlaySound("igMainMenuOptionCheckBoxOff");
    end;

	-- Get Tabard Item
	if (CheckButtonInfoDressUp:GetChecked() == true) then
		TabardAddictConfig_v2.CB_INFO_EQUIP = true; -- save config setting 
		TabardAddictDressUpModel:Dress();
		TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemID);
	else
		TabardAddictConfig_v2.CB_INFO_EQUIP = false; -- save config setting
		TabardAddictDressUpModel:Undress();
		TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemID);
	end;

end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictLauncher_OnClick(self)

	PlaySound("igMainMenuOptionCheckBoxOn");
	
	if frameTabardAddict:IsShown() then
		frameTabardAddict:Hide()
	else
		frameTabardAddict:Show()
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictSearchToggle_OnClick(self)

	PlaySound("igCharacterInfoTab");
	
	-- show or hide - edit box / header text
	if TabardAddictTab1.SearchBox:IsShown() then
		-- clear search
		if (taNameFilter ~= "") then
			taNameFilter = "";
			TabardAddictTab1.SearchBox:SetText("");
			-- populate the display table - force repopulation
			TabardAddictUpdateDisplayTable = true;
			TabardAddict_PopulateDisplayTable();
			-- sort the display table
			TabardAddict_SortDisplayTable();
			-- initial display
			TabardAddictScrollBar_Update();
		end
		TabardAddictTab1.SearchBox:Hide();
		TabardAddictTab1.SearchBox.clearButton:Hide();
		TabardAddictTab1.taDescText:Show();
	else
		TabardAddictTab1.taDescText:Hide();
		TabardAddictTab1.SearchBox:Show();
	end;

end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictSearchClear_OnClick(self)

	-- clear search
	if (taNameFilter ~= "") then
		taNameFilter = "";
		TabardAddictTab1.SearchBox:SetText("");
		-- populate the display table - force repopulation
		TabardAddictUpdateDisplayTable = true;
		TabardAddict_PopulateDisplayTable();
		-- sort the display table
		TabardAddict_SortDisplayTable();
		-- initial display
		TabardAddictScrollBar_Update();
	end

	self:ClearFocus();
    self:SetFontObject("GameFontDisable");
    self.searchIcon:SetVertexColor(0.6, 0.6, 0.6);
    self.clearButton:Hide();

end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnShow(self)
	
	local tabardsNextAchievement = 0;
	
	PlaySound("igCharacterInfoOpen");
	
	-- set description text
	TabardAddictTab1.taDescText:SetText(string.format(L.TABARD_TITLE, UnitName("player")));
	-- set search widget back to default display
	TabardAddictTab1.SearchBox:Hide();
	TabardAddictTab1.SearchBox.clearButton:Hide();
	TabardAddictTab1.taDescText:Show();

	if (TabardAddictDoServerQuery == true) then
		-- reset # of equipped tabards
		tabardsEquipped = 0;
		-- query server for data
		TabardAddict_GetServerData();
		TabardAddictDoServerQuery = false;
	end;

	-- populate counts
	if (tabardsEquipped >= 30) then
		tabardsNextAchievement = "NA";
	elseif (tabardsEquipped >= 25) then
		tabardsNextAchievement = 30;
	elseif (tabardsEquipped >= 10) then
		tabardsNextAchievement = 25;
	elseif (tabardsEquipped >= 1) then
		tabardsNextAchievement = 10;
	else
		tabardsNextAchievement = 1;
	end;
	
	-- populate the display table
	TabardAddict_PopulateDisplayTable();
	-- sort the display table
	TabardAddict_SortDisplayTable();
	-- initial display
	TabardAddictScrollBar_Update();
	-- populate the suggestions table
	TabardAddict_PopulateSuggestionsTable();
	-- sort the suggestions table
	TabardAddict_SortSuggestionsTable();
	
	
	
		-- populate equipped count and next achievement text
	if (tabardsEquipped >= 30) then
		TabardAddictTab1.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS_2, tabardsEquipped));
		TabardAddictTab4.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS_2, tabardsEquipped));
	else
		TabardAddictTab1.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS, tabardsEquipped, tabardsNextAchievement));
		TabardAddictTab4.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS, tabardsEquipped, tabardsNextAchievement));
	end;
	
	-- show/hide panels
	TabardAddictTab1:Show();
	TabardAddictTab2:Hide();
	TabardAddictTab3:Hide();
	TabardAddictTab4:Hide();
	PanelTemplates_SetTab(frameTabardAddict, 1);
	TabardAddictShowTabards();
	TabardAddictHideSuggestions();
	
	--set DressUp model and position
	TabardAddictDressUpModel:SetUnit("player");
	local settings;
	local hasAlternateForm, inAlternateForm = HasAlternateForm();
	if ( (hasAlternateForm==true) and (inAlternateForm==true) ) then
		settings = taDressUpSettings[playerRaceSex.."Alt"];
		-- debug
		if (debugMode) then
			print("loading: " .. playerRaceSex .. "Alt");
		end;
	else
		settings = taDressUpSettings[playerRaceSex];
		-- debug
		if (debugMode) then
			print("loading: " .. playerRaceSex);
		end;
	end
	
	TabardAddictDressUpModel:SetPortraitZoom(settings.zoom);
	TabardAddictDressUpModel.zoomLevel  = settings.zoom;
	TabardAddictDressUpModel:SetPosition(settings.x, settings.y, settings.z);
	
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictLauncher_OnLoad(self)

	--Show/Hide character frame launcher
	frameTabardAddictCharFrameLauncher:Show();

end;


--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnHide(self)

	PlaySound("igCharacterInfoClose");
	-- debug
	if (debugMode) then
		print("TabardAddict_OnHide() called");
	end;

	-- clear search params
	if (taNameFilter ~= "") then
		taNameFilter = "";
		TabardAddictTab1.SearchBox:SetText("");
		-- populate the display table - force repopulation
		TabardAddictUpdateDisplayTable = true;
	end

	-- Hide info frame
	if (TabardAddictInfoFrame:IsShown()) then
		TabardAddictInfoFrame:Hide();
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictShowTabards()
	local line;
	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Show();
	end;
end;
--
function TabardAddictShowSuggestions()
	local line;
	for line=1,8 do
		getglobal("TabardAddictSugg"..line):Show();
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddictHideTabards()
	local line;
	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Hide();
	end;
end;
--
function TabardAddictHideSuggestions()
	local line;
	for line=1,8 do
		getglobal("TabardAddictSugg"..line):Hide();
	end;
end;
--*****************************************************************************
--*****************************************************************************
function TabardAddict_SetConfigVariables(self)

	local name = self:GetName();
	
	if (debugMode) then
		print("TabardAddict_SetConfigVariables = " .. name);
	end;

	-- play checkbox sound
	if self.checked then
		PlaySound("igMainMenuOptionCheckBoxOn");
	else 
		PlaySound("igMainMenuOptionCheckBoxOff");
	end;

    -- update the approprate temp table based on which config clicked
    if (name == 'CheckButtonAO') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonHO') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonEQ') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonTCG') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonOther') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonNA') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonPVP') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonBCF') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonWotLKF') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonATF') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonCF') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonMOPF') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonWOD') then
    	TabardAddictUpdateDisplayTable = true;
    elseif (name == 'CheckButtonSuggAch') then
    	TabardAddictUpdateSuggestTable = true;
    elseif (name == 'CheckButtonSuggQst') then
    	TabardAddictUpdateSuggestTable = true;
    elseif (name == 'CheckButtonSuggCur') then
    	TabardAddictUpdateSuggestTable = true;
    end;
    	

	if (CheckButtonAO:GetChecked() == true) then
		TabardAddictConfig_v2.CB_AO = true;
	else
		TabardAddictConfig_v2.CB_AO = false;
	end;
	if (CheckButtonHO:GetChecked() == true) then
		TabardAddictConfig_v2.CB_HO = true;
	else
		TabardAddictConfig_v2.CB_HO = false;
	end;
	if (CheckButtonEQ:GetChecked() == true) then
		TabardAddictConfig_v2.CB_EQ = true;
	else
		TabardAddictConfig_v2.CB_EQ = false;
	end;
	if (CheckButtonTCG:GetChecked() == true) then
		TabardAddictConfig_v2.CB_TCG = true;
	else
		TabardAddictConfig_v2.CB_TCG = false;
	end;
	if (CheckButtonOther:GetChecked() == true) then
		TabardAddictConfig_v2.CB_Other = true;
	else
		TabardAddictConfig_v2.CB_Other = false;
	end;
	if (CheckButtonNA:GetChecked() == true) then
		TabardAddictConfig_v2.CB_NA = true;
	else
		TabardAddictConfig_v2.CB_NA = false;
	end;
	if (CheckButtonPVP:GetChecked() == true) then
		TabardAddictConfig_v2.CB_PVP = true;
	else
		TabardAddictConfig_v2.CB_PVP = false;
	end;
	if (CheckButtonBCF:GetChecked() == true) then
		TabardAddictConfig_v2.CB_BCF = true;
	else
		TabardAddictConfig_v2.CB_BCF = false;
	end;
	if (CheckButtonWotLKF:GetChecked() == true) then
		TabardAddictConfig_v2.CB_WotLKF = true;
	else
		TabardAddictConfig_v2.CB_WotLKF = false;
	end;
	if (CheckButtonATF:GetChecked() == true) then
		TabardAddictConfig_v2.CB_ATF = true;
	else
		TabardAddictConfig_v2.CB_ATF = false;
	end;
	if (CheckButtonCF:GetChecked() == true) then
		TabardAddictConfig_v2.CB_CF = true;
	else
		TabardAddictConfig_v2.CB_CF = false;
	end;
	if (CheckButtonMOPF:GetChecked() == true) then
		TabardAddictConfig_v2.CB_MOPF = true;
	else
		TabardAddictConfig_v2.CB_MOPF = false;
	end;
	if (CheckButtonWOD:GetChecked() == true) then
		TabardAddictConfig_v2.CB_WOD = true;
	else
		TabardAddictConfig_v2.CB_WOD = false;
	end;
	if (CheckButtonSuggAch:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_ACH = true;
	else
		TabardAddictConfig_v2.CB_SUGG_ACH = false;
	end;
	if (CheckButtonSuggQst:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_QST = true;
	else
		TabardAddictConfig_v2.CB_SUGG_QST = false;
	end;
	if (CheckButtonSuggCur:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_CUR = true;
	else
		TabardAddictConfig_v2.CB_SUGG_CUR = false;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_ConfigOnEnter(self)

	local name = self:GetName();
	
	if (debugMode) then
		print("TabardAddict_ConfigOnEnter = " .. name);
	end;
	
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	-- Show tooltip based on config item name
	GameTooltip:SetText(taTabardConfigTip[name], 1.0,1.0,1.0,1,1);
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_TotalText_OnEnter(self)

	local tableRow;
	
	-- Populate Achievement Completion tooltip
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 15, -50);
	GameTooltip:SetText(YELLOW_FONT_COLOR_CODE..L.ACHV_TT_TITLE..FONT_COLOR_CODE_CLOSE);
	GameTooltip:AddLine(" ");
	
	for tableRow in ipairs(tempAchievementTooltipData)
	do
		-- Check if achievement completed
		if (tempAchievementTooltipData[tableRow][1] == true) then
			GameTooltip:AddLine(tempAchievementTooltipData[tableRow][2], GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, 1);
			GameTooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready");
		else
			GameTooltip:AddLine(tempAchievementTooltipData[tableRow][2], GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1);
			GameTooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady");
		end;
	end;
	
	GameTooltip:Show();
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnTextChanged(self, userInput)
	local text = self:GetText();

	if (debugMode) then
		print("TabardAddict_OnTextChanged: " .. text .. ", userInput: " .. tostring(userInput));
	end;

	if (text ~= "") then
		self.label:Hide();
	else
		self.label:Show();
	end

	taNameFilter = text;
	-- get rid of right and left brackets
	taNameFilter=string.gsub(taNameFilter, "%[", "");
	taNameFilter=string.gsub(taNameFilter, "%]", "");
	
	if (userInput == true) then
		-- force repopulation of display table
		TabardAddictUpdateDisplayTable = true;
		-- populate the display table
		TabardAddict_PopulateDisplayTable();
		-- sort the display table
		TabardAddict_SortDisplayTable();
		-- initial display
		TabardAddictScrollBar_Update();
	end
end

--*****************************************************************************
--*****************************************************************************
local function OnTooltipSetItem(tooltip, ...)
	
	local itemName, itemLink = tooltip:GetItem();
	-- exit if itemLink is nil - client has not cached item yet
	if (itemLink == nil) then
		return;
	end;
	-- parse itemID from link
	local _, _, _, _, itemID, _, _, _, _, _, _, _, _, _ = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
	
	-- check if need to get data from server
	if (TabardAddictDoServerQuery == true) then
		-- reset # of equipped tabards
		tabardsEquipped = 0;
		-- query server for data
		TabardAddict_GetServerData();
		TabardAddictDoServerQuery = false;
	end;
	
	-- check if itemID in tooltiptable before adding text to tooltip
	if ( TabardAddictServerData_tooltiptable[tonumber(itemID)] ~= nill ) then
		if not taLineAdded then
			local ttString = L.STR_TABARD .. L.BEEN_EQUIPPED;
			if (TabardAddictServerData_tooltiptable[tonumber(itemID)] == true) then
				ttString = ttString .. "|cff00FF00" .. L.YES .. "|r";
			elseif (TabardAddictServerData_tooltiptable[tonumber(itemID)] == false) then
				ttString = ttString .. "|cffFF0000" ..L.NO .. "|r"
			else
				ttString = ttString .. "|cff808080" ..L.NOT_TRACKED .. "|r"
			end;
			tooltip:AddLine(ttString);
			taLineAdded = true;
		end;
	end; 
end;

--*****************************************************************************
--*****************************************************************************
local function OnTooltipCleared(tooltip, ...)
	taLineAdded = false;
end;

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnLoad(self)

	if (debugMode) then
		print("TabardAddict_OnLoad: "..self:GetName());
	end;

	-- Register the frame so it can be dragged
	self:RegisterForDrag("LeftButton");

	-- Register Events
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UPDATE_FACTION");
	
	-- HookScripts for in-game tooltips
	GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem);
	GameTooltip:HookScript("OnTooltipCleared", OnTooltipCleared);
	
	-- init variables
	TabardAddictDoServerQuery = true;
	TabardAddictUpdateDisplayTable = true;
	TabardAddictUpdateSuggestTable = true;
	
	-- Tab Handling code
	PanelTemplates_SetNumTabs(self, 4);
	PanelTemplates_SetTab(self, 1);
	
	-- show/hide panels
	TabardAddictTab1:Show();
	TabardAddictTab2:Hide();
	TabardAddictTab3:Hide();
	TabardAddictTab4:Hide();
	
	-- update portrait emblem
	SetPortraitToTexture(self.taTabardEmblem, "Interface\\ICONS\\INV_Chest_Cloth_30")
	
	self.taTitleText:SetText(L.ADDON_TITLE);
	TabardAddictTab3.taVersionText:SetText("v"..TABARD_ADDICT_VERSION);
	
	-- set addon text
	self.Tab1Label:SetText(L.TAB1_LABEL);
	self.Tab2Label:SetText(L.TAB2_LABEL);
	self.Tab3Label:SetText(L.TAB3_LABEL);
	self.Tab4Label:SetText(L.TAB4_LABEL);
	
	TabardAddictTab2.taConfigDescText:SetText(L.CONFIG_TITLE);
	TabardAddictTab2.fsTabardTypes:SetText(L.CONFIG_HDR1);
	TabardAddictTab2.fsTabardFilter:SetText(L.CONFIG_HDR2);
	TabardAddictTab2.fsTabardSugg:SetText(L.CONFIG_HDR3);
	CheckButtonAO.fs:SetText(L.CONFIG_AO);
	CheckButtonHO.fs:SetText(L.CONFIG_HO);
	CheckButtonEQ.fs:SetText(L.CONFIG_EQ);
	CheckButtonTCG.fs:SetText(L.CONFIG_TCG);
	CheckButtonPVP.fs:SetText(L.CONFIG_PVP);
	CheckButtonBCF.fs:SetText(L.CONFIG_BCF);
	CheckButtonWotLKF.fs:SetText(L.CONFIG_WOTLK);
	CheckButtonATF.fs:SetText(L.CONFIG_ATF);
	CheckButtonCF.fs:SetText(L.CONFIG_CF);
	CheckButtonMOPF.fs:SetText(L.CONFIG_MOPF);
	CheckButtonWOD.fs:SetText(L.CONFIG_WOD);
	CheckButtonNA.fs:SetText(L.CONFIG_NA);
	CheckButtonOther.fs:SetText(L.CONFIG_OTHER);
	CheckButtonSuggAch.fs:SetText(L.CONFIG_SUGG_ACH);
	CheckButtonSuggQst.fs:SetText(L.CONFIG_SUGG_QST);
	CheckButtonSuggCur.fs:SetText(L.CONFIG_SUGG_CUR);
	
	TabardAddictTab3.taAboutDescText:SetText(L.ABOUT_TITLE);
	TabardAddictTab3.hdr1:SetText(L.ABOUT_HDR1);
	TabardAddictAboutInfo:SetText(L.ABOUT_TXT);
	
	TabardAddictTab4.taSuggestDescText:SetText(L.SUGGESTIONS_TITLE);
	
	-- Check Alliance/Horde/Neutral
	TabardAddict_CheckPlayerFaction();
	
	-- init name filter
	taNameFilter = "";
	TabardAddictTab1.SearchBox.label:SetText(SEARCH);

	-- make closable with escape
	tinsert(UISpecialFrames,self:GetName());
end;


